@@ -43,7 +43,7 @@ static const char *x86DisassemblerGetInstrName(unsigned Opcode)
 #ifndef NDEBUG
 #define debug(s) do { x86DisassemblerDebug(__FILE__, __LINE__, s); } while (0)
 #else
-#define debug(s) do { } while (0)
+#define debug(s) ((void) 0)
 #endif
 
 /*
@@ -54,7 +54,7 @@ static const char *x86DisassemblerGetInstrName(unsigned Opcode)
  * @return          - The InstructionContext to use when looking up an
  *                    an instruction with these attributes.
  */
-static InstructionContext contextForAttrs(uint8_t attrMask)
+static InstructionContext contextForAttrs(uint16_t attrMask)
 {
 	return CONTEXTS_SYM[attrMask];
 }
@@ -72,42 +72,49 @@ static InstructionContext contextForAttrs(uint8_t attrMask)
  */
 static int modRMRequired(OpcodeType type,
 		InstructionContext insnContext,
-		uint8_t opcode)
+		uint16_t opcode)
 {
-	const struct ContextDecision* decision = 0;
+	const struct OpcodeDecision* decision = NULL;
+	const uint8_t *indextable = NULL;
+	uint8_t index;
 
 	switch (type) {
 		case ONEBYTE:
-			decision = &ONEBYTE_SYM;
+			decision = ONEBYTE_SYM;
+			indextable = index_x86DisassemblerOneByteOpcodes;
 			break;
 		case TWOBYTE:
-			decision = &TWOBYTE_SYM;
+			decision = TWOBYTE_SYM;
+			indextable = index_x86DisassemblerTwoByteOpcodes;
 			break;
 		case THREEBYTE_38:
-			decision = &THREEBYTE38_SYM;
+			decision = THREEBYTE38_SYM;
+			indextable = index_x86DisassemblerThreeByte38Opcodes;
 			break;
 		case THREEBYTE_3A:
-			decision = &THREEBYTE3A_SYM;
-			break;
-		case THREEBYTE_A6:
-			decision = &THREEBYTEA6_SYM;
-			break;
-		case THREEBYTE_A7:
-			decision = &THREEBYTEA7_SYM;
+			decision = THREEBYTE3A_SYM;
+			indextable = index_x86DisassemblerThreeByte3AOpcodes;
 			break;
 		case XOP8_MAP:
-			decision = &XOP8_MAP_SYM;
+			decision = XOP8_MAP_SYM;
+			indextable = index_x86DisassemblerXOP8Opcodes;
 			break;
 		case XOP9_MAP:
-			decision = &XOP9_MAP_SYM;
+			decision = XOP9_MAP_SYM;
+			indextable = index_x86DisassemblerXOP9Opcodes;
 			break;
 		case XOPA_MAP:
-			decision = &XOPA_MAP_SYM;
+			decision = XOPA_MAP_SYM;
+			indextable = index_x86DisassemblerXOPAOpcodes;
 			break;
 	}
 
-	return decision->opcodeDecisions[insnContext].modRMDecisions[opcode].
-		modrm_type != MODRM_ONEENTRY;
+	index = indextable[insnContext];
+	if (index)
+		return decision[index - 1].modRMDecisions[opcode].
+			modrm_type != MODRM_ONEENTRY;
+	else
+		return false;
 }
 
 /*
@@ -126,34 +133,65 @@ static InstrUID decode(OpcodeType type,
 		uint8_t modRM)
 {
 	const struct ModRMDecision* dec = 0;
+	const uint8_t *indextable = NULL;
+	uint8_t index;
 
 	switch (type) {
 		case ONEBYTE:
-			dec = &ONEBYTE_SYM.opcodeDecisions[insnContext].modRMDecisions[opcode];
+			indextable = index_x86DisassemblerOneByteOpcodes;
+			index = indextable[insnContext];
+			if (index)
+				dec = &ONEBYTE_SYM[index - 1].modRMDecisions[opcode];
+			else
+				dec = &emptyTable.modRMDecisions[opcode];
 			break;
 		case TWOBYTE:
-			dec = &TWOBYTE_SYM.opcodeDecisions[insnContext].modRMDecisions[opcode];
+			indextable = index_x86DisassemblerTwoByteOpcodes;
+			index = indextable[insnContext];
+			if (index)
+				dec = &TWOBYTE_SYM[index - 1].modRMDecisions[opcode];
+			else
+				dec = &emptyTable.modRMDecisions[opcode];
 			break;
 		case THREEBYTE_38:
-			dec = &THREEBYTE38_SYM.opcodeDecisions[insnContext].modRMDecisions[opcode];
+			indextable = index_x86DisassemblerThreeByte38Opcodes;
+			index = indextable[insnContext];
+			if (index)
+				dec = &THREEBYTE38_SYM[index - 1].modRMDecisions[opcode];
+			else
+				dec = &emptyTable.modRMDecisions[opcode];
 			break;
 		case THREEBYTE_3A:
-			dec = &THREEBYTE3A_SYM.opcodeDecisions[insnContext].modRMDecisions[opcode];
-			break;
-		case THREEBYTE_A6:
-			dec = &THREEBYTEA6_SYM.opcodeDecisions[insnContext].modRMDecisions[opcode];
-			break;
-		case THREEBYTE_A7:
-			dec = &THREEBYTEA7_SYM.opcodeDecisions[insnContext].modRMDecisions[opcode];
+			indextable = index_x86DisassemblerThreeByte3AOpcodes;
+			index = indextable[insnContext];
+			if (index)
+				dec = &THREEBYTE3A_SYM[index - 1].modRMDecisions[opcode];
+			else
+				dec = &emptyTable.modRMDecisions[opcode];
 			break;
 		case XOP8_MAP:
-			dec = &XOP8_MAP_SYM.opcodeDecisions[insnContext].modRMDecisions[opcode];
+			indextable = index_x86DisassemblerXOP8Opcodes;
+			index = indextable[insnContext];
+			if (index)
+				dec = &XOP8_MAP_SYM[index - 1].modRMDecisions[opcode];
+			else
+				dec = &emptyTable.modRMDecisions[opcode];
 			break;
 		case XOP9_MAP:
-			dec = &XOP9_MAP_SYM.opcodeDecisions[insnContext].modRMDecisions[opcode];
+			indextable = index_x86DisassemblerXOP9Opcodes;
+			index = indextable[insnContext];
+			if (index)
+				dec = &XOP9_MAP_SYM[index - 1].modRMDecisions[opcode];
+			else
+				dec = &emptyTable.modRMDecisions[opcode];
 			break;
 		case XOPA_MAP:
-			dec = &XOPA_MAP_SYM.opcodeDecisions[insnContext].modRMDecisions[opcode];
+			indextable = index_x86DisassemblerXOPAOpcodes;
+			index = indextable[insnContext];
+			if (index)
+				dec = &XOPA_MAP_SYM[index - 1].modRMDecisions[opcode];
+			else
+				dec = &emptyTable.modRMDecisions[opcode];
 			break;
 	}
 
@@ -464,55 +502,57 @@ static int readPrefixes(struct InternalInstruction* insn)
 			dbgprintf(insn, "Found prefix 0x%hhx", byte);
 	}
 
-	insn->vexXopType = TYPE_NO_VEX_XOP;
+	insn->vectorExtensionType = TYPE_NO_VEX_XOP;
 
-	if (byte == 0xc4) {
-		uint8_t byte1;
+	if (byte == 0x62) {
+		uint8_t byte1, byte2;
 
-		if (lookAtByte(insn, &byte1)) {
-			dbgprintf(insn, "Couldn't read second byte of VEX");
+		if (consumeByte(insn, &byte1)) {
+			dbgprintf(insn, "Couldn't read second byte of EVEX prefix");
 			return -1;
 		}
 
-		if (insn->mode == MODE_64BIT || (byte1 & 0xc0) == 0xc0) {
-			insn->vexXopType = TYPE_VEX_3B;
-			insn->necessaryPrefixLocation = insn->readerCursor - 1;
-		}
-		else {
-			unconsumeByte(insn);
-			insn->necessaryPrefixLocation = insn->readerCursor - 1;
+		if (lookAtByte(insn, &byte2)) {
+			dbgprintf(insn, "Couldn't read third byte of EVEX prefix");
+			return -1;
 		}
 
-		if (insn->vexXopType == TYPE_VEX_3B) {
-			insn->vexXopPrefix[0] = byte;
-			consumeByte(insn, &insn->vexXopPrefix[1]);
-			consumeByte(insn, &insn->vexXopPrefix[2]);
+		if ((insn->mode == MODE_64BIT || (byte1 & 0xc0) == 0xc0) &&
+				((~byte1 & 0xc) == 0xc) && ((byte2 & 0x4) == 0x4)) {
+			insn->vectorExtensionType = TYPE_EVEX;
+		}
+		else {
+			unconsumeByte(insn); /* unconsume byte1 */
+			unconsumeByte(insn); /* unconsume byte  */
+			insn->necessaryPrefixLocation = insn->readerCursor - 2;
+		}
+
+		if (insn->vectorExtensionType == TYPE_EVEX) {
+			insn->vectorExtensionPrefix[0] = byte;
+			insn->vectorExtensionPrefix[1] = byte1;
+			if (consumeByte(insn, &insn->vectorExtensionPrefix[2])) {
+				dbgprintf(insn, "Couldn't read third byte of EVEX prefix");
+				return -1;
+			}
+			if (consumeByte(insn, &insn->vectorExtensionPrefix[3])) {
+				dbgprintf(insn, "Couldn't read fourth byte of EVEX prefix");
+				return -1;
+			}
 
 			/* We simulate the REX prefix for simplicity's sake */
-
 			if (insn->mode == MODE_64BIT) {
 				insn->rexPrefix = 0x40
-					| (wFromVEX3of3(insn->vexXopPrefix[2]) << 3)
-					| (rFromVEX2of3(insn->vexXopPrefix[1]) << 2)
-					| (xFromVEX2of3(insn->vexXopPrefix[1]) << 1)
-					| (bFromVEX2of3(insn->vexXopPrefix[1]) << 0);
+					| (wFromEVEX3of4(insn->vectorExtensionPrefix[2]) << 3)
+					| (rFromEVEX2of4(insn->vectorExtensionPrefix[1]) << 2)
+					| (xFromEVEX2of4(insn->vectorExtensionPrefix[1]) << 1)
+					| (bFromEVEX2of4(insn->vectorExtensionPrefix[1]) << 0);
 			}
 
-			switch (ppFromVEX3of3(insn->vexXopPrefix[2]))
-			{
-				default:
-					break;
-				case VEX_PREFIX_66:
-					hasOpSize = TRUE;
-					break;
-			}
-
-			dbgprintf(insn, "Found VEX prefix 0x%hhx 0x%hhx 0x%hhx",
-					insn->vexXopPrefix[0], insn->vexXopPrefix[1],
-					insn->vexXopPrefix[2]);
+			dbgprintf(insn, "Found EVEX prefix 0x%hhx 0x%hhx 0x%hhx 0x%hhx",
+					insn->vectorExtensionPrefix[0], insn->vectorExtensionPrefix[1],
+					insn->vectorExtensionPrefix[2], insn->vectorExtensionPrefix[3]);
 		}
-	}
-	else if (byte == 0xc5) {
+	} else if (byte == 0xc4) {
 		uint8_t byte1;
 
 		if (lookAtByte(insn, &byte1)) {
@@ -521,42 +561,70 @@ static int readPrefixes(struct InternalInstruction* insn)
 		}
 
 		if (insn->mode == MODE_64BIT || (byte1 & 0xc0) == 0xc0) {
-			insn->vexXopType = TYPE_VEX_2B;
+			insn->vectorExtensionType = TYPE_VEX_3B;
+			insn->necessaryPrefixLocation = insn->readerCursor - 1;
 		}
 		else {
 			unconsumeByte(insn);
+			insn->necessaryPrefixLocation = insn->readerCursor - 1;
 		}
 
-		if (insn->vexXopType == TYPE_VEX_2B) {
-			insn->vexXopPrefix[0] = byte;
-			consumeByte(insn, &insn->vexXopPrefix[1]);
+		if (insn->vectorExtensionType == TYPE_VEX_3B) {
+			insn->vectorExtensionPrefix[0] = byte;
+			consumeByte(insn, &insn->vectorExtensionPrefix[1]);
+			consumeByte(insn, &insn->vectorExtensionPrefix[2]);
+
+			/* We simulate the REX prefix for simplicity's sake */
+			if (insn->mode == MODE_64BIT) {
+				insn->rexPrefix = 0x40
+					| (wFromVEX3of3(insn->vectorExtensionPrefix[2]) << 3)
+					| (rFromVEX2of3(insn->vectorExtensionPrefix[1]) << 2)
+					| (xFromVEX2of3(insn->vectorExtensionPrefix[1]) << 1)
+					| (bFromVEX2of3(insn->vectorExtensionPrefix[1]) << 0);
+
+			}
+		}
+	} else if (byte == 0xc5) {
+		uint8_t byte1;
+
+		if (lookAtByte(insn, &byte1)) {
+			dbgprintf(insn, "Couldn't read second byte of VEX");
+			return -1;
+		}
+
+		if (insn->mode == MODE_64BIT || (byte1 & 0xc0) == 0xc0) {
+			insn->vectorExtensionType = TYPE_VEX_2B;
+		} else {
+			unconsumeByte(insn);
+		}
+
+		if (insn->vectorExtensionType == TYPE_VEX_2B) {
+			insn->vectorExtensionPrefix[0] = byte;
+			consumeByte(insn, &insn->vectorExtensionPrefix[1]);
 
 			if (insn->mode == MODE_64BIT) {
 				insn->rexPrefix = 0x40
-					| (rFromVEX2of2(insn->vexXopPrefix[1]) << 2);
+					| (rFromVEX2of2(insn->vectorExtensionPrefix[1]) << 2);
 			}
 
-			switch (ppFromVEX2of2(insn->vexXopPrefix[1]))
-			{
+			switch (ppFromVEX2of2(insn->vectorExtensionPrefix[1])) {
 				default:
 					break;
 				case VEX_PREFIX_66:
 					hasOpSize = TRUE;
 					break;
 			}
-
-			dbgprintf(insn, "Found VEX prefix 0x%hhx 0x%hhx", insn->vexXopPrefix[0], insn->vexXopPrefix[1]);
 		}
 	} else if (byte == 0x8f) {
 		uint8_t byte1;
 
 		if (lookAtByte(insn, &byte1)) {
-			dbgprintf(insn, "Couldn't read second byte of XOP");
+			// dbgprintf(insn, "Couldn't read second byte of XOP");
 			return -1;
 		}
 
 		if ((byte1 & 0x38) != 0x0) { /* 0 in these 3 bits is a POP instruction. */
-			insn->vexXopType = TYPE_XOP;
+			insn->vectorExtensionType = TYPE_XOP;
 			insn->necessaryPrefixLocation = insn->readerCursor - 1;
 		}
 		else {
@@ -564,33 +632,27 @@ static int readPrefixes(struct InternalInstruction* insn)
 			insn->necessaryPrefixLocation = insn->readerCursor - 1;
 		}
 
-		if (insn->vexXopType == TYPE_XOP) {
-			insn->vexXopPrefix[0] = byte;
-			consumeByte(insn, &insn->vexXopPrefix[1]);
-			consumeByte(insn, &insn->vexXopPrefix[2]);
+		if (insn->vectorExtensionType == TYPE_XOP) {
+			insn->vectorExtensionPrefix[0] = byte;
+			consumeByte(insn, &insn->vectorExtensionPrefix[1]);
+			consumeByte(insn, &insn->vectorExtensionPrefix[2]);
 
 			/* We simulate the REX prefix for simplicity's sake */
-
 			if (insn->mode == MODE_64BIT) {
 				insn->rexPrefix = 0x40
-					| (wFromXOP3of3(insn->vexXopPrefix[2]) << 3)
-					| (rFromXOP2of3(insn->vexXopPrefix[1]) << 2)
-					| (xFromXOP2of3(insn->vexXopPrefix[1]) << 1)
-					| (bFromXOP2of3(insn->vexXopPrefix[1]) << 0);
+					| (wFromXOP3of3(insn->vectorExtensionPrefix[2]) << 3)
+					| (rFromXOP2of3(insn->vectorExtensionPrefix[1]) << 2)
+					| (xFromXOP2of3(insn->vectorExtensionPrefix[1]) << 1)
+					| (bFromXOP2of3(insn->vectorExtensionPrefix[1]) << 0);
 			}
 
-			switch (ppFromXOP3of3(insn->vexXopPrefix[2]))
-			{
+			switch (ppFromXOP3of3(insn->vectorExtensionPrefix[2])) {
 				default:
 					break;
 				case VEX_PREFIX_66:
 					hasOpSize = TRUE;
 					break;
 			}
-
-			dbgprintf(insn, "Found XOP prefix 0x%hhx 0x%hhx 0x%hhx",
-					insn->vexXopPrefix[0], insn->vexXopPrefix[1],
-					insn->vexXopPrefix[2]);
 		}
 	} else {
 		if (insn->mode == MODE_64BIT) {
@@ -598,14 +660,14 @@ static int readPrefixes(struct InternalInstruction* insn)
 				uint8_t opcodeByte;
 
 				if (lookAtByte(insn, &opcodeByte) || ((opcodeByte & 0xf0) == 0x40)) {
-					dbgprintf(insn, "Redundant REX prefix");
+					// dbgprintf(insn, "Redundant REX prefix");
 					return -1;
 				}
 
 				insn->rexPrefix = byte;
 				insn->necessaryPrefixLocation = insn->readerCursor - 2;
 
-				dbgprintf(insn, "Found REX prefix 0x%hhx", byte);
+				// dbgprintf(insn, "Found REX prefix 0x%hhx", byte);
 			} else {
 				unconsumeByte(insn);
 				insn->necessaryPrefixLocation = insn->readerCursor - 1;
@@ -632,6 +694,11 @@ static int readPrefixes(struct InternalInstruction* insn)
 			insn->addressSize        = (hasAdSize ? 4 : 8);
 			insn->displacementSize   = 4;
 			insn->immediateSize      = 4;
+		} else if (insn->rexPrefix) {
+			insn->registerSize       = (hasOpSize ? 2 : 4);
+			insn->addressSize        = (hasAdSize ? 4 : 8);
+			insn->displacementSize   = (hasOpSize ? 2 : 4);
+			insn->immediateSize      = (hasOpSize ? 2 : 4);
 		} else {
 			insn->registerSize       = (hasOpSize ? 2 : 4);
 			insn->addressSize        = (hasAdSize ? 4 : 8);
@@ -659,13 +726,28 @@ static int readOpcode(struct InternalInstruction* insn)
 
 	insn->opcodeType = ONEBYTE;
 
-	if (insn->vexXopType == TYPE_VEX_3B)
-	{
-		switch (mmmmmFromVEX2of3(insn->vexXopPrefix[1]))
-		{
+	if (insn->vectorExtensionType == TYPE_EVEX) {
+		switch (mmFromEVEX2of4(insn->vectorExtensionPrefix[1])) {
 			default:
-				dbgprintf(insn, "Unhandled m-mmmm field for instruction (0x%hhx)",
-						mmmmmFromVEX2of3(insn->vexXopPrefix[1]));
+				// dbgprintf(insn, "Unhandled mm field for instruction (0x%hhx)",
+				// 		mmFromEVEX2of4(insn->vectorExtensionPrefix[1]));
+				return -1;
+			case VEX_LOB_0F:
+				insn->opcodeType = TWOBYTE;
+				return consumeByte(insn, &insn->opcode);
+			case VEX_LOB_0F38:
+				insn->opcodeType = THREEBYTE_38;
+				return consumeByte(insn, &insn->opcode);
+			case VEX_LOB_0F3A:
+				insn->opcodeType = THREEBYTE_3A;
+				return consumeByte(insn, &insn->opcode);
+		}
+	} else if (insn->vectorExtensionType == TYPE_VEX_3B) {
+					switch (mmmmmFromVEX2of3(insn->vectorExtensionPrefix[1])) {
+
+			default:
+				// dbgprintf(insn, "Unhandled m-mmmm field for instruction (0x%hhx)",
+				//		mmmmmFromVEX2of3(insn->vectorExtensionPrefix[1]));
 				return -1;
 			case VEX_LOB_0F:
 				insn->twoByteEscape = 0x0f;
@@ -682,20 +764,15 @@ static int readOpcode(struct InternalInstruction* insn)
 				insn->opcodeType = THREEBYTE_3A;
 				return consumeByte(insn, &insn->opcode);
 		}
-	}
-	else if (insn->vexXopType == TYPE_VEX_2B)
-	{
+	} else if (insn->vectorExtensionType == TYPE_VEX_2B) {
 		insn->twoByteEscape = 0x0f;
 		insn->opcodeType = TWOBYTE;
 		return consumeByte(insn, &insn->opcode);
-	}
-	else if (insn->vexXopType == TYPE_XOP)
-	{
-		switch (mmmmmFromXOP2of3(insn->vexXopPrefix[1]))
-		{
+	} else if (insn->vectorExtensionType == TYPE_XOP) {
+		switch (mmmmmFromXOP2of3(insn->vectorExtensionPrefix[1])) {
 			default:
-				dbgprintf(insn, "Unhandled m-mmmm field for instruction (0x%hhx)",
-						mmmmmFromVEX2of3(insn->vexXopPrefix[1]));
+				// dbgprintf(insn, "Unhandled m-mmmm field for instruction (0x%hhx)",
+				// 		mmmmmFromVEX2of3(insn->vectorExtensionPrefix[1]));
 				return -1;
 			case XOP_MAP_SELECT_8:
 				// FIXME: twoByteEscape?
@@ -716,7 +793,7 @@ static int readOpcode(struct InternalInstruction* insn)
 		return -1;
 
 	if (current == 0x0f) {
-		dbgprintf(insn, "Found a two-byte escape prefix (0x%hhx)", current);
+		// dbgprintf(insn, "Found a two-byte escape prefix (0x%hhx)", current);
 
 		insn->twoByteEscape = current;
 
@@ -724,7 +801,7 @@ static int readOpcode(struct InternalInstruction* insn)
 			return -1;
 
 		if (current == 0x38) {
-			dbgprintf(insn, "Found a three-byte escape prefix (0x%hhx)", current);
+			// dbgprintf(insn, "Found a three-byte escape prefix (0x%hhx)", current);
 
 			insn->threeByteEscape = current;
 
@@ -733,7 +810,7 @@ static int readOpcode(struct InternalInstruction* insn)
 
 			insn->opcodeType = THREEBYTE_38;
 		} else if (current == 0x3a) {
-			dbgprintf(insn, "Found a three-byte escape prefix (0x%hhx)", current);
+			// dbgprintf(insn, "Found a three-byte escape prefix (0x%hhx)", current);
 
 			insn->threeByteEscape = current;
 
@@ -741,26 +818,8 @@ static int readOpcode(struct InternalInstruction* insn)
 				return -1;
 
 			insn->opcodeType = THREEBYTE_3A;
-		} else if (current == 0xa6) {
-			dbgprintf(insn, "Found a three-byte escape prefix (0x%hhx)", current);
-
-			insn->threeByteEscape = current;
-
-			if (consumeByte(insn, &current))
-				return -1;
-
-			insn->opcodeType = THREEBYTE_A6;
-		} else if (current == 0xa7) {
-			dbgprintf(insn, "Found a three-byte escape prefix (0x%hhx)", current);
-
-			insn->threeByteEscape = current;
-
-			if (consumeByte(insn, &current))
-				return -1;
-
-			insn->opcodeType = THREEBYTE_A7;
 		} else {
-			dbgprintf(insn, "Didn't find a three-byte escape prefix");
+			// dbgprintf(insn, "Didn't find a three-byte escape prefix");
 
 			insn->opcodeType = TWOBYTE;
 		}
@@ -792,17 +851,17 @@ static int readModRM(struct InternalInstruction* insn);
  */
 static int getIDWithAttrMask(uint16_t* instructionID,
 		struct InternalInstruction* insn,
-		uint8_t attrMask)
+		uint16_t attrMask)
 {
 	BOOL hasModRMExtension;
 
-	uint8_t instructionClass;
+	uint16_t instructionClass;
 
 	instructionClass = contextForAttrs(attrMask);
 
 	hasModRMExtension = modRMRequired(insn->opcodeType,
 			instructionClass,
-			insn->opcode);
+			insn->opcode) == TRUE;
 
 	if (hasModRMExtension) {
 		if (readModRM(insn))
@@ -831,7 +890,7 @@ static int getIDWithAttrMask(uint16_t* instructionID,
  */
 static BOOL is16BitEquivalent(const char* orig, const char* equiv)
 {
-	off_t i;
+	size_t i;
 
 	for (i = 0;; i++) {
 		if (orig[i] == '\0' && equiv[i] == '\0')
@@ -861,8 +920,9 @@ static BOOL is16BitEquivalent(const char* orig, const char* equiv)
  */
 static int getID(struct InternalInstruction* insn)
 {
-	uint8_t attrMask;
+	uint16_t attrMask;
 	uint16_t instructionID;
+	const struct InstructionSpecifier *spec;
 
 	//printf(">>> getID()\n");
 
@@ -871,11 +931,11 @@ static int getID(struct InternalInstruction* insn)
 	if (insn->mode == MODE_64BIT)
 		attrMask |= ATTR_64BIT;
 
-	if (insn->vexXopType != TYPE_NO_VEX_XOP) {
-		attrMask |= ATTR_VEX;
+	if (insn->vectorExtensionType != TYPE_NO_VEX_XOP) {
+		attrMask |= (insn->vectorExtensionType == TYPE_EVEX) ? ATTR_EVEX : ATTR_VEX;
 
-		if (insn->vexXopType == TYPE_VEX_3B) {
-			switch (ppFromVEX3of3(insn->vexXopPrefix[2])) {
+		if (insn->vectorExtensionType == TYPE_EVEX) {
+			switch (ppFromEVEX3of4(insn->vectorExtensionPrefix[2])) {
 				case VEX_PREFIX_66:
 					attrMask |= ATTR_OPSIZE;
 					break;
@@ -887,11 +947,54 @@ static int getID(struct InternalInstruction* insn)
 					break;
 			}
 
-			if (lFromVEX3of3(insn->vexXopPrefix[2]))
+			if (zFromEVEX4of4(insn->vectorExtensionPrefix[3]))
+				attrMask |= ATTR_EVEXKZ;
+			if (bFromEVEX4of4(insn->vectorExtensionPrefix[3]))
+				attrMask |= ATTR_EVEXB;
+			if (aaaFromEVEX4of4(insn->vectorExtensionPrefix[3]))
+				attrMask |= ATTR_EVEXK;
+			if (lFromEVEX4of4(insn->vectorExtensionPrefix[3]))
+				attrMask |= ATTR_EVEXL;
+			if (l2FromEVEX4of4(insn->vectorExtensionPrefix[3]))
+				attrMask |= ATTR_EVEXL2;
+		}
+		else if (insn->vectorExtensionType == TYPE_VEX_3B) {
+			switch (ppFromVEX3of3(insn->vectorExtensionPrefix[2])) {
+
+				case VEX_PREFIX_66:
+					attrMask |= ATTR_OPSIZE;
+					break;
+				case VEX_PREFIX_F3:
+					attrMask |= ATTR_XS;
+					break;
+				case VEX_PREFIX_F2:
+					attrMask |= ATTR_XD;
+					break;
+			}
+
+			if (lFromVEX3of3(insn->vectorExtensionPrefix[2]))
+
+				attrMask |= ATTR_VEXL;
+		} else if (insn->vectorExtensionType == TYPE_VEX_2B) {
+			switch (ppFromVEX2of2(insn->vectorExtensionPrefix[1])) {
+
+				case VEX_PREFIX_66:
+					attrMask |= ATTR_OPSIZE;
+					break;
+				case VEX_PREFIX_F3:
+					attrMask |= ATTR_XS;
+					break;
+				case VEX_PREFIX_F2:
+					attrMask |= ATTR_XD;
+					break;
+			}
+
+			if (lFromVEX2of2(insn->vectorExtensionPrefix[1]))
 				attrMask |= ATTR_VEXL;
 		}
-		else if (insn->vexXopType == TYPE_VEX_2B) {
-			switch (ppFromVEX2of2(insn->vexXopPrefix[1])) {
+		else if (insn->vectorExtensionType == TYPE_XOP) {
+			switch (ppFromXOP3of3(insn->vectorExtensionPrefix[2])) {
+
 				case VEX_PREFIX_66:
 					attrMask |= ATTR_OPSIZE;
 					break;
@@ -903,23 +1006,7 @@ static int getID(struct InternalInstruction* insn)
 					break;
 			}
 
-			if (lFromVEX2of2(insn->vexXopPrefix[1]))
-				attrMask |= ATTR_VEXL;
-		}
-		else if (insn->vexXopType == TYPE_XOP) {
-			switch (ppFromXOP3of3(insn->vexXopPrefix[2])) {
-				case VEX_PREFIX_66:
-					attrMask |= ATTR_OPSIZE;
-					break;
-				case VEX_PREFIX_F3:
-					attrMask |= ATTR_XS;
-					break;
-				case VEX_PREFIX_F2:
-					attrMask |= ATTR_XD;
-					break;
-			}
-
-			if (lFromXOP3of3(insn->vexXopPrefix[2]))
+			if (lFromXOP3of3(insn->vectorExtensionPrefix[2]))
 				attrMask |= ATTR_VEXL;
 		}
 		else {
@@ -927,7 +1014,7 @@ static int getID(struct InternalInstruction* insn)
 		}
 	}
 	else {
-		if (isPrefixAtLocation(insn, 0x66, insn->necessaryPrefixLocation))
+		if (insn->mode != MODE_16BIT && isPrefixAtLocation(insn, 0x66, insn->necessaryPrefixLocation))
 			attrMask |= ATTR_OPSIZE;
 		else if (isPrefixAtLocation(insn, 0x67, insn->necessaryPrefixLocation))
 			attrMask |= ATTR_ADSIZE;
@@ -937,24 +1024,33 @@ static int getID(struct InternalInstruction* insn)
 			attrMask |= ATTR_XD;
 	}
 
-	// FIXME: hack to fix problem in 16bit mode with data size override
-	if (insn->mode == MODE_16BIT) {
-		if (attrMask & ATTR_OPSIZE)
-			attrMask &= ~ATTR_OPSIZE;
-		else
-			attrMask |= ATTR_OPSIZE;
-	}
-
 	if (insn->rexPrefix & 0x08)
 		attrMask |= ATTR_REXW;
 
 	if (getIDWithAttrMask(&instructionID, insn, attrMask))
 		return -1;
 
-	/* The following clauses compensate for limitations of the tables. */
+	/*
+	 * JCXZ/JECXZ need special handling for 16-bit mode because the meaning
+	 * of the AdSize prefix is inverted w.r.t. 32-bit mode.
+	 */
+	if (insn->mode == MODE_16BIT && insn->opcode == 0xE3) {
+		spec = specifierForUID(instructionID);
 
-	// FIXME: dirty hack for 16bit data size override
-	if (insn->mode != MODE_16BIT && insn->prefixPresent[0x66] && !(attrMask & ATTR_OPSIZE)) {
+		/*
+		 * Check for Ii8PCRel instructions. We could alternatively do a
+		 * string-compare on the names, but this is probably cheaper.
+		 */
+		if (x86OperandSets[spec->operands][0].type == TYPE_REL8) {
+			attrMask ^= ATTR_ADSIZE;
+			if (getIDWithAttrMask(&instructionID, insn, attrMask))
+				return -1;
+		}
+	}
+
+	/* The following clauses compensate for limitations of the tables. */
+	if ((insn->mode == MODE_16BIT || insn->prefixPresent[0x66]) &&
+			!(attrMask & ATTR_OPSIZE)) {
 		/*
 		 * The instruction tables make no distinction between instructions that
 		 * allow OpSize anywhere (i.e., 16-bit operations) and that need it in a
@@ -986,7 +1082,8 @@ static int getID(struct InternalInstruction* insn)
 		specWithOpSizeName =
 			x86DisassemblerGetInstrName(instructionIDWithOpsize);
 
-		if (is16BitEquivalent(specName, specWithOpSizeName)) {
+		if (is16BitEquivalent(specName, specWithOpSizeName) &&
+				(insn->mode == MODE_16BIT) ^ insn->prefixPresent[0x66]) {
 			insn->instructionID = instructionIDWithOpsize;
 			insn->spec = specifierForUID(instructionIDWithOpsize);
 		} else {
@@ -1052,7 +1149,7 @@ static int readSIB(struct InternalInstruction* insn)
 	SIBBase sibBaseBase = 0;
 	uint8_t index, base;
 
-	dbgprintf(insn, "readSIB()");
+	// dbgprintf(insn, "readSIB()");
 
 	if (insn->consumedSIB)
 		return 0;
@@ -1061,7 +1158,7 @@ static int readSIB(struct InternalInstruction* insn)
 
 	switch (insn->addressSize) {
 		case 2:
-			dbgprintf(insn, "SIB-based addressing doesn't work in 16-bit mode");
+			// dbgprintf(insn, "SIB-based addressing doesn't work in 16-bit mode");
 			return -1;
 			break;
 		case 4:
@@ -1078,6 +1175,8 @@ static int readSIB(struct InternalInstruction* insn)
 		return -1;
 
 	index = indexFromSIB(insn->sib) | (xFromREX(insn->rexPrefix) << 3);
+	if (insn->vectorExtensionType == TYPE_EVEX)
+		index |= v2FromEVEX4of4(insn->vectorExtensionPrefix[3]) << 4;
 
 	switch (index) {
 		case 0x4:
@@ -1110,6 +1209,7 @@ static int readSIB(struct InternalInstruction* insn)
 
 	switch (base) {
 		case 0x5:
+		case 0xd:
 			switch (modFromModRM(insn->modRM)) {
 				case 0x0:
 					insn->eaDisplacement = EA_DISP_32;
@@ -1117,13 +1217,11 @@ static int readSIB(struct InternalInstruction* insn)
 					break;
 				case 0x1:
 					insn->eaDisplacement = EA_DISP_8;
-					insn->sibBase = (insn->addressSize == 4 ?
-							SIB_BASE_EBP : SIB_BASE_RBP);
+					insn->sibBase = (SIBBase)(sibBaseBase + base);
 					break;
 				case 0x2:
 					insn->eaDisplacement = EA_DISP_32;
-					insn->sibBase = (insn->addressSize == 4 ?
-							SIB_BASE_EBP : SIB_BASE_RBP);
+					insn->sibBase = (SIBBase)(sibBaseBase + base);
 					break;
 				case 0x3:
 					debug("Cannot have Mod = 0b11 and a SIB byte");
@@ -1151,13 +1249,13 @@ static int readDisplacement(struct InternalInstruction* insn)
 	int16_t d16;
 	int32_t d32;
 
-	dbgprintf(insn, "readDisplacement()");
+	// dbgprintf(insn, "readDisplacement()");
 
 	if (insn->consumedDisplacement)
 		return 0;
 
 	insn->consumedDisplacement = TRUE;
-	insn->displacementOffset = insn->readerCursor - insn->startLocation;
+	insn->displacementOffset = (uint8_t)(insn->readerCursor - insn->startLocation);
 
 	switch (insn->eaDisplacement) {
 		case EA_DISP_NONE:
@@ -1195,7 +1293,7 @@ static int readModRM(struct InternalInstruction* insn)
 {
 	uint8_t mod, rm, reg;
 
-	dbgprintf(insn, "readModRM()");
+	// dbgprintf(insn, "readModRM()");
 
 	if (insn->consumedModRM)
 		return 0;
@@ -1230,6 +1328,10 @@ static int readModRM(struct InternalInstruction* insn)
 
 	reg |= rFromREX(insn->rexPrefix) << 3;
 	rm  |= bFromREX(insn->rexPrefix) << 3;
+	if (insn->vectorExtensionType == TYPE_EVEX) {
+		reg |= r2FromEVEX2of4(insn->vectorExtensionPrefix[1]) << 4;
+		rm  |=  xFromEVEX2of4(insn->vectorExtensionPrefix[1]) << 4;
+	}
 
 	insn->reg = (Reg)(insn->regBase + reg);
 
@@ -1252,6 +1354,7 @@ static int readModRM(struct InternalInstruction* insn)
 				case 0x1:
 					insn->eaBase = (EABase)(insn->eaBaseBase + rm);
 					insn->eaDisplacement = EA_DISP_8;
+					insn->displacementSize = 1;
 					if (readDisplacement(insn))
 						return -1;
 					break;
@@ -1276,6 +1379,7 @@ static int readModRM(struct InternalInstruction* insn)
 				case 0x0:
 					insn->eaDisplacement = EA_DISP_NONE; /* readSIB may override this */
 					switch (rm) {
+						case 0x14:
 						case 0x4:
 						case 0xc:   /* in case REXW.b is set */
 							insn->eaBase = (insn->addressSize == 4 ?
@@ -1294,11 +1398,15 @@ static int readModRM(struct InternalInstruction* insn)
 							insn->eaBase = (EABase)(insn->eaBaseBase + rm);
 							break;
 					}
+
 					break;
 				case 0x1:
+					insn->displacementSize = 1;
+					/* FALLTHROUGH */
 				case 0x2:
 					insn->eaDisplacement = (mod == 0x1 ? EA_DISP_8 : EA_DISP_32);
 					switch (rm) {
+						case 0x14:
 						case 0x4:
 						case 0xc:   /* in case REXW.b is set */
 							insn->eaBase = EA_BASE_sib;
@@ -1381,13 +1489,13 @@ static int readModRM(struct InternalInstruction* insn)
 			*valid = 0;                                         \
 			return 0;                                           \
 			case TYPE_Rv:                                         \
-																  return base + index;                                \
+																  return (uint8_t)(base + index);                     \
 			case TYPE_R8:                                         \
 																  if (insn->rexPrefix &&                              \
-																		  index >= 4 && index <= 7) {                      \
-																	  return prefix##_SPL + (index - 4);                \
+																		  index >= 4 && index <= 7) {                 \
+																	  return prefix##_SPL + (index - 4);              \
 																  } else {                                            \
-																	  return prefix##_AL + index;                       \
+																	  return prefix##_AL + index;                     \
 																  }                                                   \
 			case TYPE_R16:                                        \
 																  return prefix##_AX + index;                         \
@@ -1404,6 +1512,10 @@ static int readModRM(struct InternalInstruction* insn)
 			case TYPE_XMM32:                                      \
 			case TYPE_XMM:                                        \
 																  return prefix##_XMM0 + index;                       \
+			case TYPE_VK1:                                        \
+			case TYPE_VK8:                                        \
+			case TYPE_VK16:                                       \
+																  return prefix##_K0 + index;                         \
 			case TYPE_MM64:                                       \
 			case TYPE_MM32:                                       \
 			case TYPE_MM:                                         \
@@ -1455,7 +1567,7 @@ static int fixupReg(struct InternalInstruction *insn,
 {
 	uint8_t valid;
 
-	dbgprintf(insn, "fixupReg()");
+	// dbgprintf(insn, "fixupReg()");
 
 	switch ((OperandEncoding)op->encoding) {
 		default:
@@ -1472,7 +1584,7 @@ static int fixupReg(struct InternalInstruction *insn,
 		case ENCODING_REG:
 			insn->reg = (Reg)fixupRegValue(insn,
 					(OperandType)op->type,
-					insn->reg - insn->regBase,
+					(uint8_t)(insn->reg - insn->regBase),
 					&valid);
 			if (!valid)
 				return -1;
@@ -1481,7 +1593,7 @@ static int fixupReg(struct InternalInstruction *insn,
 			if (insn->eaBase >= insn->eaRegBase) {
 				insn->eaBase = (EABase)fixupRMValue(insn,
 						(OperandType)op->type,
-						insn->eaBase - insn->eaRegBase,
+						(uint8_t)(insn->eaBase - insn->eaRegBase),
 						&valid);
 				if (!valid)
 					return -1;
@@ -1492,46 +1604,12 @@ static int fixupReg(struct InternalInstruction *insn,
 	return 0;
 }
 
-/*
- * readOpcodeModifier - Reads an operand from the opcode field of an
- *   instruction.  Handles AddRegFrm instructions.
- *
- * @param insn    - The instruction whose opcode field is to be read.
- * @param inModRM - Indicates that the opcode field is to be read from the
- *                  ModR/M extension; useful for escape opcodes
- * @return        - 0 on success; nonzero otherwise.
- */
-static int readOpcodeModifier(struct InternalInstruction* insn)
-{
-	dbgprintf(insn, "readOpcodeModifier()");
-
-	if (insn->consumedOpcodeModifier)
-		return 0;
-
-	insn->consumedOpcodeModifier = TRUE;
-
-	switch (insn->spec->modifierType) {
-		default:
-			debug("Unknown modifier type.");
-			return -1;
-		case MODIFIER_NONE:
-			debug("No modifier but an operand expects one.");
-			return -1;
-		case MODIFIER_OPCODE:
-			insn->opcodeModifier = insn->opcode - insn->spec->modifierBase;
-			return 0;
-		case MODIFIER_MODRM:
-			insn->opcodeModifier = insn->modRM - insn->spec->modifierBase;
-			return 0;
-	}
-}
-
 /*
  * readOpcodeRegister - Reads an operand from the opcode field of an
  *   instruction and interprets it appropriately given the operand width.
  *   Handles AddRegFrm instructions.
  *
- * @param insn  - See readOpcodeModifier().
+ * @param insn  - the instruction whose opcode field is to be read.
  * @param size  - The width (in bytes) of the register being specified.
  *                1 means AL and friends, 2 means AX, 4 means EAX, and 8 means
  *                RAX.
@@ -1539,10 +1617,7 @@ static int readOpcodeModifier(struct InternalInstruction* insn)
  */
 static int readOpcodeRegister(struct InternalInstruction* insn, uint8_t size)
 {
-	dbgprintf(insn, "readOpcodeRegister()");
-
-	if (readOpcodeModifier(insn))
-		return -1;
+	// dbgprintf(insn, "readOpcodeRegister()");
 
 	if (size == 0)
 		size = insn->registerSize;
@@ -1552,7 +1627,7 @@ static int readOpcodeRegister(struct InternalInstruction* insn, uint8_t size)
 	switch (size) {
 		case 1:
 			insn->opcodeRegister = (Reg)(MODRM_REG_AL + ((bFromREX(insn->rexPrefix) << 3)
-						| insn->opcodeModifier));
+						| (insn->opcode & 7)));
 			if (insn->rexPrefix &&
 					insn->opcodeRegister >= MODRM_REG_AL + 0x4 &&
 					insn->opcodeRegister < MODRM_REG_AL + 0x8) {
@@ -1564,17 +1639,17 @@ static int readOpcodeRegister(struct InternalInstruction* insn, uint8_t size)
 		case 2:
 			insn->opcodeRegister = (Reg)(MODRM_REG_AX
 					+ ((bFromREX(insn->rexPrefix) << 3)
-						| insn->opcodeModifier));
+						| (insn->opcode & 7)));
 			break;
 		case 4:
 			insn->opcodeRegister = (Reg)(MODRM_REG_EAX
 					+ ((bFromREX(insn->rexPrefix) << 3)
-						| insn->opcodeModifier));
+						| (insn->opcode & 7)));
 			break;
 		case 8:
 			insn->opcodeRegister = (Reg)(MODRM_REG_RAX
 					+ ((bFromREX(insn->rexPrefix) << 3)
-						| insn->opcodeModifier));
+						| (insn->opcode & 7)));
 			break;
 	}
 
@@ -1597,7 +1672,7 @@ static int readImmediate(struct InternalInstruction* insn, uint8_t size)
 	uint32_t imm32;
 	uint64_t imm64;
 
-	dbgprintf(insn, "readImmediate()");
+	// dbgprintf(insn, "readImmediate()");
 
 	if (insn->numImmediatesConsumed == 2) {
 		debug("Already consumed two immediates");
@@ -1608,7 +1683,7 @@ static int readImmediate(struct InternalInstruction* insn, uint8_t size)
 		size = insn->immediateSize;
 	else
 		insn->immediateSize = size;
-	insn->immediateOffset = insn->readerCursor - insn->startLocation;
+	insn->immediateOffset = (uint8_t)(insn->readerCursor - insn->startLocation);
 
 	switch (size) {
 		case 1:
@@ -1647,14 +1722,16 @@ static int readImmediate(struct InternalInstruction* insn, uint8_t size)
  */
 static int readVVVV(struct InternalInstruction* insn)
 {
-	dbgprintf(insn, "readVVVV()");
+	// dbgprintf(insn, "readVVVV()");
 
-	if (insn->vexXopType == TYPE_VEX_3B)
-		insn->vvvv = vvvvFromVEX3of3(insn->vexXopPrefix[2]);
-	else if (insn->vexXopType == TYPE_VEX_2B)
-		insn->vvvv = vvvvFromVEX2of2(insn->vexXopPrefix[1]);
-	else if (insn->vexXopType == TYPE_XOP)
-		insn->vvvv = vvvvFromXOP3of3(insn->vexXopPrefix[2]);
+	if (insn->vectorExtensionType == TYPE_EVEX)
+		insn->vvvv = vvvvFromEVEX3of4(insn->vectorExtensionPrefix[2]);
+	else if (insn->vectorExtensionType == TYPE_VEX_3B)
+		insn->vvvv = vvvvFromVEX3of3(insn->vectorExtensionPrefix[2]);
+	else if (insn->vectorExtensionType == TYPE_VEX_2B)
+		insn->vvvv = vvvvFromVEX2of2(insn->vectorExtensionPrefix[1]);
+	else if (insn->vectorExtensionType == TYPE_XOP)
+		insn->vvvv = vvvvFromXOP3of3(insn->vectorExtensionPrefix[2]);
 	else
 		return -1;
 
@@ -1664,6 +1741,25 @@ static int readVVVV(struct InternalInstruction* insn)
 	return 0;
 }
 
+/*
+ * readMaskRegister - Reads an mask register from the opcode field of an
+ *   instruction.
+ *
+ * @param insn    - The instruction whose opcode field is to be read.
+ * @return        - 0 on success; nonzero otherwise.
+ */
+static int readMaskRegister(struct InternalInstruction* insn)
+{
+	// dbgprintf(insn, "readMaskRegister()");
+
+	if (insn->vectorExtensionType != TYPE_EVEX)
+		return -1;
+
+	insn->writemask = aaaFromEVEX4of4(insn->vectorExtensionPrefix[3]);
+
+	return 0;
+}
+	 
 /*
  * readOperands - Consults the specifier for an instruction and consumes all
  *   operands for that instruction, interpreting them as it goes.
@@ -1686,6 +1782,8 @@ static int readOperands(struct InternalInstruction* insn)
 		//printf(">>> encoding[%u] = %u\n", index, x86OperandSets[insn->spec->operands][index].encoding);
 		switch (x86OperandSets[insn->spec->operands][index].encoding) {
 			case ENCODING_NONE:
+			case ENCODING_SI:
+			case ENCODING_DI:
 				break;
 			case ENCODING_REG:
 			case ENCODING_RM:
@@ -1700,7 +1798,7 @@ static int readOperands(struct InternalInstruction* insn)
 			case ENCODING_CP:
 			case ENCODING_CO:
 			case ENCODING_CT:
-				dbgprintf(insn, "We currently don't hande code-offset encodings");
+				// dbgprintf(insn, "We currently don't hande code-offset encodings");
 				return -1;
 			case ENCODING_IB:
 				if (sawRegImm) {
@@ -1728,14 +1826,8 @@ static int readOperands(struct InternalInstruction* insn)
 					return -1;
 				break;
 			case ENCODING_ID:
-				// FIXME: dirty hack for 16bit call, which is not properly supported so far.
-				if (insn->mode == MODE_16BIT) {
-					if (readImmediate(insn, 2))
-						return -1;
-				} else {
-					if (readImmediate(insn, 4))
-						return -1;
-				}
+				if (readImmediate(insn, 4))
+					return -1;
 				break;
 			case ENCODING_IO:
 				if (readImmediate(insn, 8))
@@ -1769,9 +1861,7 @@ static int readOperands(struct InternalInstruction* insn)
 				if (readOpcodeRegister(insn, 0))
 					return -1;
 				break;
-			case ENCODING_I:
-				if (readOpcodeModifier(insn))
-					return -1;
+			case ENCODING_FP:
 				break;
 			case ENCODING_VVVV:
 				needVVVV = 0; /* Mark that we have found a VVVV operand. */
@@ -1780,10 +1870,14 @@ static int readOperands(struct InternalInstruction* insn)
 				if (fixupReg(insn, &x86OperandSets[insn->spec->operands][index]))
 					return -1;
 				break;
+			case ENCODING_WRITEMASK:
+				if (readMaskRegister(insn))
+					return -1;
+				break;
 			case ENCODING_DUP:
 				break;
 			default:
-				dbgprintf(insn, "Encountered an operand with an unknown encoding.");
+				// dbgprintf(insn, "Encountered an operand with an unknown encoding.");
 				return -1;
 		}
 	}
@@ -1841,13 +1935,29 @@ int decodeInstruction(struct InternalInstruction* insn,
 
 	insn->operands = &x86OperandSets[insn->spec->operands][0];
 
-	insn->length = insn->readerCursor - insn->startLocation;
+	insn->length = (size_t)(insn->readerCursor - insn->startLocation);
 
-	dbgprintf(insn, "Read from 0x%llx to 0x%llx: length %zu",
-			startLoc, insn->readerCursor, insn->length);
+	// dbgprintf(insn, "Read from 0x%llx to 0x%llx: length %zu",
+	// 		startLoc, insn->readerCursor, insn->length);
 
 	if (insn->length > 15)
 		dbgprintf(insn, "Instruction exceeds 15-byte limit");
 
+#if 0
+	printf("\n>>> x86OperandSets = %lu\n", sizeof(x86OperandSets));
+	printf(">>> x86DisassemblerInstrSpecifiers = %lu\n", sizeof(x86DisassemblerInstrSpecifiers));
+	printf(">>> x86DisassemblerContexts = %lu\n", sizeof(x86DisassemblerContexts));
+	printf(">>> modRMTable = %lu\n", sizeof(modRMTable));
+	printf(">>> x86DisassemblerOneByteOpcodes = %lu\n", sizeof(x86DisassemblerOneByteOpcodes));
+	printf(">>> x86DisassemblerTwoByteOpcodes = %lu\n", sizeof(x86DisassemblerTwoByteOpcodes));
+	printf(">>> x86DisassemblerThreeByte38Opcodes = %lu\n", sizeof(x86DisassemblerThreeByte38Opcodes));
+	printf(">>> x86DisassemblerThreeByte3AOpcodes = %lu\n", sizeof(x86DisassemblerThreeByte3AOpcodes));
+	printf(">>> x86DisassemblerThreeByteA6Opcodes = %lu\n", sizeof(x86DisassemblerThreeByteA6Opcodes));
+	printf(">>> x86DisassemblerThreeByteA7Opcodes= %lu\n", sizeof(x86DisassemblerThreeByteA7Opcodes));
+	printf(">>> x86DisassemblerXOP8Opcodes = %lu\n", sizeof(x86DisassemblerXOP8Opcodes));
+	printf(">>> x86DisassemblerXOP9Opcodes = %lu\n", sizeof(x86DisassemblerXOP9Opcodes));
+	printf(">>> x86DisassemblerXOPAOpcodes = %lu\n\n", sizeof(x86DisassemblerXOPAOpcodes));
+#endif
+
 	return 0;
 }
